@@ -1,11 +1,30 @@
+# Copyright 2009-2013 Justin Riley
+#
+# This file is part of StarCluster.
+#
+# StarCluster is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option) any
+# later version.
+#
+# StarCluster is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with StarCluster. If not, see <http://www.gnu.org/licenses/>.
+
 """
 clustersetup.py
 """
 import posixpath
 
+from starcluster import utils
 from starcluster import threadpool
 from starcluster.utils import print_timing
 from starcluster.logger import log
+from starcluster import exception
 
 
 class ClusterSetup(object):
@@ -52,6 +71,18 @@ class ClusterSetup(object):
         """
         raise NotImplementedError('run method not implemented')
 
+    def __new__(typ, *args, **kwargs):
+        """
+        DO NOT OVERRIDE!
+
+        This is an internal method used for plugin accounting.
+        Do not override! If you *must* don't forget to call super!
+        """
+        plugin = super(ClusterSetup, typ).__new__(typ)
+        plugin_class_name = utils.get_fq_class_name(plugin)
+        plugin.__plugin_metadata__ = (plugin_class_name, args, kwargs)
+        return plugin
+
 
 class DefaultClusterSetup(ClusterSetup):
     """
@@ -149,7 +180,14 @@ class DefaultClusterSetup(ClusterSetup):
         the new user to be the existing uid/gid of the dir in EBS rather than
         chowning potentially terabytes of data.
         """
+        user = user or self._user
         uid, gid = self._get_new_user_id(user)
+        if uid == 0 or gid == 0:
+            raise exception.BaseException(
+                "Cannot create user: {0:s} (uid: {1:1d}, gid: {2:1d}). This "
+                "is caused by /home/{0:s} directory being owned by root. To "
+                "fix this you'll need to create a new AMI. Note that the "
+                "instance is still up.".format(user, uid, gid))
         log.info("Creating cluster user: %s (uid: %d, gid: %d)" %
                  (user, uid, gid))
         self._add_user_to_nodes(uid, gid, self._nodes)
@@ -237,7 +275,7 @@ class DefaultClusterSetup(ClusterSetup):
         """
         # setup /etc/fstab on master to use block device if specified
         master = self._master
-        devs = master.ssh.ls('/dev')
+        devices = master.get_device_map()
         for vol in self._volumes:
             vol = self._volumes[vol]
             vol_id = vol.get("volume_id")
@@ -247,44 +285,40 @@ class DefaultClusterSetup(ClusterSetup):
             if not (vol_id and device and mount_path):
                 log.error("missing required settings for vol %s" % vol)
                 continue
-            dev_exists = master.ssh.path_exists(device)
-            if not dev_exists and device.startswith('/dev/sd'):
+            if not device in devices and device.startswith('/dev/sd'):
                 # check for "correct" device in unpatched kernels
                 device = device.replace('/dev/sd', '/dev/xvd')
-                dev_exists = master.ssh.path_exists(device)
-            if not dev_exists:
-                log.warn("Cannot find device %s for volume %s" %
-                         (device, vol_id))
-                log.warn("Not mounting %s on %s" % (vol_id, mount_path))
-                log.warn("This usually means there was a problem "
-                         "attaching the EBS volume to the master node")
-                continue
+                if not device in devices:
+                    log.warn("Cannot find device %s for volume %s" %
+                             (device, vol_id))
+                    log.warn("Not mounting %s on %s" % (vol_id, mount_path))
+                    log.warn("This usually means there was a problem "
+                             "attaching the EBS volume to the master node")
+                    continue
+            partitions = master.get_partition_map(device=device)
             if not volume_partition:
-                partitions = filter(lambda x: x.startswith(device), devs)
-                if len(partitions) == 1:
+                if len(partitions) == 0:
                     volume_partition = device
-                elif len(partitions) == 2:
-                    volume_partition = device + '1'
+                elif len(partitions) == 1:
+                    volume_partition = partitions.popitem()[0]
                 else:
                     log.error(
                         "volume has more than one partition, please specify "
                         "which partition to use (e.g. partition=0, "
                         "partition=1, etc.) in the volume's config")
                     continue
-            elif not master.ssh.path_exists(volume_partition):
+            elif not volume_partition in partitions:
                 log.warn("Cannot find partition %s on volume %s" %
                          (volume_partition, vol_id))
-                log.warn("Not mounting %s on %s" % (vol_id,
-                                                    mount_path))
+                log.warn("Not mounting %s on %s" % (vol_id, mount_path))
                 log.warn("This either means that the volume has not "
-                         "been partitioned or that the partition"
+                         "been partitioned or that the partition "
                          "specified does not exist on the volume")
                 continue
             log.info("Mounting EBS volume %s on %s..." % (vol_id, mount_path))
-            mount_map = self._master.get_mount_map()
-            dev = mount_map.get(volume_partition)
-            if dev:
-                path, fstype, options = dev
+            mount_map = master.get_mount_map()
+            if volume_partition in mount_map:
+                path, fstype, options = mount_map.get(volume_partition)
                 if path != mount_path:
                     log.error("Volume %s is mounted on %s, not on %s" %
                               (vol_id, path, mount_path))
@@ -293,7 +327,7 @@ class DefaultClusterSetup(ClusterSetup):
                         "Volume %s already mounted on %s...skipping" %
                         (vol_id, mount_path))
                 continue
-            self._master.mount_device(volume_partition, mount_path)
+            master.mount_device(volume_partition, mount_path)
 
     def _get_nfs_export_paths(self):
         export_paths = ['/home']
@@ -335,21 +369,18 @@ class DefaultClusterSetup(ClusterSetup):
 
     def run(self, nodes, master, user, user_shell, volumes):
         """Start cluster configuration"""
-        try:
-            self._nodes = nodes
-            self._master = master
-            self._user = user
-            self._user_shell = user_shell
-            self._volumes = volumes
-            self._setup_hostnames()
-            self._setup_ebs_volumes()
-            self._setup_cluster_user()
-            self._setup_scratch()
-            self._setup_etc_hosts()
-            self._setup_nfs()
-            self._setup_passwordless_ssh()
-        finally:
-            self.pool.shutdown()
+        self._nodes = nodes
+        self._master = master
+        self._user = user
+        self._user_shell = user_shell
+        self._volumes = volumes
+        self._setup_hostnames()
+        self._setup_ebs_volumes()
+        self._setup_cluster_user()
+        self._setup_scratch()
+        self._setup_etc_hosts()
+        self._setup_nfs()
+        self._setup_passwordless_ssh()
 
     def _remove_from_etc_hosts(self, node):
         nodes = filter(lambda x: x.id != node.id, self.running_nodes)
