@@ -1,3 +1,20 @@
+# Copyright 2009-2013 Justin Riley
+#
+# This file is part of StarCluster.
+#
+# StarCluster is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option) any
+# later version.
+#
+# StarCluster is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with StarCluster. If not, see <http://www.gnu.org/licenses/>.
+
 import os
 import sys
 import base64
@@ -109,7 +126,8 @@ class CmdShell(CmdBase):
                                       'ipcontroller-client.json')
                 if cl.master_node.ssh.isfile(json):
                     log.info("Fetching connector file from cluster...")
-                    os.makedirs(ipcluster_dir)
+                    if not os.path.exists(ipcluster_dir):
+                        os.makedirs(ipcluster_dir)
                     cl.master_node.ssh.get(json, local_json)
                 else:
                     self.parser.error(
@@ -119,15 +137,16 @@ class CmdShell(CmdBase):
             key_location = cl.master_node.key_location
             self._add_to_known_hosts(cl.master_node)
             log.info("Loading parallel IPython client and view")
-            rc = Client(local_json, sshkey=key_location, packer='pickle')
+            rc = Client(local_json, sshkey=key_location)
             local_ns['Client'] = Client
             local_ns['ipcluster'] = cl
             local_ns['ipclient'] = rc
             local_ns['ipview'] = rc[:]
         modules = [(starcluster.__name__ + '.' + module, module)
                    for module in starcluster.__all__]
-        modules += [('boto', 'boto'), ('ssh', 'ssh'),
-                    ('workerpool', 'workerpool'), ('jinja2', 'jinja2')]
+        modules += [('boto', 'boto'), ('paramiko', 'paramiko'),
+                    ('workerpool', 'workerpool'), ('jinja2', 'jinja2'),
+                    ('Crypto', 'Crypto'), ('iptools', 'iptools')]
         for fullname, modname in modules:
             log.info('Importing module %s' % modname)
             try:
